==> utils-1.3.8.jar: utils/rabbit_service.clj <==
(ns utils.rabbit-service
  (:require [langohr.core :as rmq]
            [langohr.channel :as lch]
            [langohr.queue :as lq]
            [langohr.basic :as lb]
            [langohr.consumers :as lc]
            [langohr.exchange :as le]
            [cheshire.core :as json]
            [utils.logger :as logger]))

(def ^{:const true} default-exchange-name "")

(defn connection-settings [host-env username-env password-env]
  {:username (System/getenv username-env)
   :password (System/getenv password-env)
   :vhost    "/"
   :host     (System/getenv host-env)
   :port     5672})

(def q-connection-setting (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))

(defn parse-rabbit-payload [payload]
  (json/parse-string (String. payload "UTF-8") true))

(defn message-handler-wrapper [raw-handler ch {:keys [content-type delivery-tag type routing-key] :as meta} payload]
  (raw-handler (parse-rabbit-payload payload)))

(defn message-rpc-handler-wrapper [raw-handler ch {:keys [content-type delivery-tag type delivery-tag reply-to correlation-id] :as meta} payload]
  (let [response (raw-handler (parse-rabbit-payload payload))]
    (lb/publish ch "" reply-to (json/generate-string response) {:correlation-id correlation-id})
    (lb/ack ch delivery-tag)))


(defn start-consumer
  [queue-name message-handler]
  (try
    (let [conn (rmq/connect (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))
          ch (lch/open conn)
          callback (partial message-handler-wrapper message-handler)]
      (lq/declare ch queue-name {:exclusive false :auto-delete false :durable true})
      (lc/subscribe ch queue-name callback {:auto-ack true}))
    (catch Throwable th (logger/debug th))))

;;TODO - Use exhanges
;;TODO - What if something dies on the queue...
(defn start-rpc-consumer
  [queue-name message-handler]
  (try
    (let [conn (rmq/connect (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))
          ch (lch/open conn)
          callback (partial message-rpc-handler-wrapper message-handler)]
      (lq/declare ch queue-name {:auto-delete false})
      (lb/qos ch 1)
      (lc/blocking-subscribe ch queue-name callback))
    (catch Throwable th (logger/debug th))))

(defn publish
  [message queue-name]
  (try
    (let [conn (rmq/connect (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))
          ch (lch/open conn)]
      (lq/declare ch queue-name {:auto-delete false :durable true :exclusive false})
      (lb/publish ch default-exchange-name queue-name message)
      (Thread/sleep 100)
      (rmq/close ch)
      (rmq/close conn))
    (catch Throwable th (logger/debug th))))

(defn correlation-id-equals?
  [correlation-id d]
  (= (.getCorrelationId (.getProperties d)) correlation-id))

;;TODO - Setup like an exchange
;;TODO - How to setup a timeout
(defn publish-rpc
  [message queue-name]
  (let [conn     (rmq/connect (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))
          ch       (lch/open conn)
          cbq      (lq/declare ch "" {:auto-delete false :exclusive true})
          consumer (lc/create-queueing ch {})]
    (let [correlation-id (str (java.util.UUID/randomUUID))]
      (lb/publish ch "" queue-name message {:reply-to (:queue cbq) :correlation-id correlation-id})
      (lb/consume ch (:queue cbq) consumer)
      (-> (first (filter (partial correlation-id-equals? correlation-id)
                    ;;TODO - Submit patch to add the timeout for .nextDelivery
                     (lc/deliveries-seq consumer)))
          .getBody
          (parse-rabbit-payload)))))

(defn publish->exchange
  ([type exchange-name routing-key message]
   (publish->exchange type exchange-name routing-key message {:content-type "application/json"}))
  ([type exchange-name routing-key message headers]
   (try
     (let [conn (rmq/connect (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))
           ch (lch/open conn)]
       (le/declare ch exchange-name type {:auto-delete false :durable true :exclusive false})
       (lb/publish ch exchange-name routing-key (json/generate-string message) headers)
       (Thread/sleep 100)
       (rmq/close ch)
       (rmq/close conn))
     (catch Throwable th (logger/error th)))))

(defn bind->exchange
  "when passing a routing information for a routing or topic exchange it should be a map with one or more key value pairs:
  i.e. {'orange-route' (fn [...] (do-something. . .)) }"
  [& {:keys [x-name x-type queue-name topics handler] :or {queue-name "" topics [""]}}]
  (try
    (let [conn (rmq/connect q-connection-setting)
          ch (lch/open conn)
          queue-name' (:queue (lq/declare ch queue-name {:auto-delete false :durable true :exclusive true}))]
      (le/declare ch x-name x-type {:auto-delete false :durable true :exclusive false})
      (doseq [topic topics]
        (lq/bind ch queue-name' x-name {:routing-key topic}))
      (lc/subscribe ch queue-name' handler))
    (catch Throwable th (logger/error th))))

==> utils-1.4.2.jar: utils/rabbit_service.clj <==
(ns utils.rabbit-service
  (:require [langohr.core :as rmq]
            [langohr.channel :as lch]
            [langohr.queue :as lq]
            [langohr.basic :as lb]
            [langohr.consumers :as lc]
            [langohr.exchange :as le]
            [cheshire.core :as json]
            [utils.logger :as logger])
  (:import [com.rabbitmq.client Channel Consumer DefaultConsumer QueueingConsumer
            QueueingConsumer$Delivery ShutdownSignalException Envelope
            AMQP$BasicProperties QueueingConsumer$Delivery]))

(def ^{:const true} default-exchange-name "")

(defn connection-settings [host-env username-env password-env]
  {:username (System/getenv username-env)
   :password (System/getenv password-env)
   :vhost    "/"
   :host     (System/getenv host-env)
   :port     5672})

(def q-connection-setting (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))

(defn deliveries-seq
  "Builds a lazy seq of delivery instances from a queueing consumer."
  [^QueueingConsumer qcs timeoutms]
  (lazy-seq (cons (.nextDelivery qcs timeoutms) (deliveries-seq qcs timeoutms))))

(defn parse-rabbit-payload [payload]
  (if (nil? payload)
    nil
    (json/parse-string (String. payload "UTF-8") true)))

(defn message-handler-wrapper [raw-handler ch {:keys [content-type delivery-tag type routing-key] :as meta} payload]
  (raw-handler (parse-rabbit-payload payload)))

(defn message-rpc-handler-wrapper [raw-handler ch {:keys [content-type delivery-tag type delivery-tag reply-to correlation-id] :as meta} payload]
  (let [response (raw-handler (parse-rabbit-payload payload))]
    (lb/publish ch "" reply-to (json/generate-string response) {:correlation-id correlation-id})
    (lb/ack ch delivery-tag)))


(defn start-consumer
  [queue-name message-handler]
  (try
    (let [conn (rmq/connect (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))
          ch (lch/open conn)
          callback (partial message-handler-wrapper message-handler)]
      (lq/declare ch queue-name {:exclusive false :auto-delete false :durable true})
      (lc/subscribe ch queue-name callback {:auto-ack true}))
    (catch Throwable th (logger/debug th))))

;;TODO - Use exhanges
;;TODO - What if something dies on the queue...
(defn start-rpc-consumer
  [queue-name message-handler]
  (try
    (let [conn (rmq/connect (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))
          ch (lch/open conn)
          callback (partial message-rpc-handler-wrapper message-handler)]
      (lq/declare ch queue-name {:auto-delete false})
      (lb/qos ch 1)
      (lc/blocking-subscribe ch queue-name callback))
    (catch Throwable th (logger/debug th))))

(defn publish
  [message queue-name]
  (try
    (let [conn (rmq/connect (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))
          ch (lch/open conn)]
      (lq/declare ch queue-name {:auto-delete false :durable true :exclusive false})
      (lb/publish ch default-exchange-name queue-name message)
      (Thread/sleep 100)
      (rmq/close ch)
      (rmq/close conn))
    (catch Throwable th (logger/debug th))))

(defn correlation-id-equals?
  [correlation-id d]
  (= (.getCorrelationId (.getProperties d)) correlation-id))

(defn nil-or-correlation-id-equals?
  [correlation-id d]
  (or (nil? d)
      (= (.getCorrelationId (.getProperties d)) correlation-id)))

(defn nil-or-body
  [d]
  (if (nil? d)
    nil
    (.getBody d)))

;;TODO - Setup like an exchange
;;TODO - How to setup a timeout
(defn publish-rpc
  [message queue-name timeoutms]
  (let [conn     (rmq/connect (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))
          ch       (lch/open conn)
          cbq      (lq/declare ch "" {:auto-delete false :exclusive true})
          consumer (lc/create-queueing ch {})]
    (let [correlation-id (str (java.util.UUID/randomUUID))]
      (lb/publish ch "" queue-name message {:reply-to (:queue cbq) :correlation-id correlation-id})
      (lb/consume ch (:queue cbq) consumer)
      (-> (->> (deliveries-seq consumer timeoutms) 
               (filter (partial nil-or-correlation-id-equals? correlation-id))
               (first))
          nil-or-body
          (parse-rabbit-payload)))))

(defn publish->exchange
  ([type exchange-name routing-key message]
   (publish->exchange type exchange-name routing-key message {:content-type "application/json"}))
  ([type exchange-name routing-key message headers]
   (try
     (let [conn (rmq/connect (connection-settings "AMQPHOST" "AMQPUSER" "AMQPPASS"))
           ch (lch/open conn)]
       (le/declare ch exchange-name type {:auto-delete false :durable true :exclusive false})
       (lb/publish ch exchange-name routing-key (json/generate-string message) headers)
       (Thread/sleep 100)
       (rmq/close ch)
       (rmq/close conn))
     (catch Throwable th (logger/error th)))))

(defn bind->exchange
  [& {:keys [x-name x-type queue-name topics handler] :or {queue-name "" topics [""]}}]
  (try
    (let [conn (rmq/connect q-connection-setting)
          ch (lch/open conn)
          queue-name' (:queue (lq/declare ch queue-name {:auto-delete false :durable true :exclusive true}))]
      (le/declare ch x-name x-type {:auto-delete false :durable true :exclusive false})
      (doseq [topic topics]
        (lq/bind ch queue-name' x-name {:routing-key topic}))
      (lc/subscribe ch queue-name' handler))
    (catch Throwable th (logger/error th))))
